// Copyright 2009-2012 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Dynamic;
using System.Data;
using System.Data.OleDb;

namespace DynamicDB {
  public class DynamicField 
    //: DynamicObject 
  {
    public DynamicField(DynamicDatabase db, DynamicTable table, string fieldName) {
      this.db = db;
      this.table = table;
      this.fieldName = fieldName;
    }

    DynamicDatabase db;
    DynamicTable table;
    string fieldName;

    private bool? isNumeric = null;
    public bool IsNumeric {
      get {
        if (isNumeric == null)
          isNumeric = CalcIsNumeric( );
        return isNumeric.Value; 
      }
    }

    bool CalcIsNumeric( ) {
      var dataType = table.DataTable.Columns[fieldName].DataType;
      return 
        dataType == typeof(System.Int32) ||
        dataType == typeof(System.Int16) ||
        dataType == typeof(System.Int64) ||
        dataType == typeof(System.Double) ||
        dataType == typeof(System.Single) ||
        dataType == typeof(System.Decimal) ;
    }

    //public override bool TryInvokeMember(InvokeMemberBinder binder, object[] args, out object result) {
    //  if (IsNumeric) {
    //    if (binder.Name == "Sum") {
    //      result = CalcSum( );
    //      return true;
    //    }
    //    else if (binder.Name == "Avg") {
    //      result = CalcAvg( );
    //      return true;
    //    }
    //  }

    //  return base.TryInvokeMember(binder, args, out result);
    //}

    private object CalcAggregate(string aggregateFunc) {
      var query = new OleDbCommand(String.Format("select {0}({1}) from {2}",
              aggregateFunc, fieldName, table.Name), db.Connection);
      return query.ExecuteScalar( );
    }

    private object CalcSum( ) {
      return CalcAggregate("sum");
    }

    private object CalcAvg( ) {
      return CalcAggregate("avg");
    }











    // For the UseFromPython demo, this method is required (as long as 
    // this class derives from DynamicObject)

    //public override bool TryGetMember(GetMemberBinder binder, out object result) {
    //  if (IsNumeric) {
    //    if (binder.Name == "Sum") {
    //      result = (Func<object>) CalcSum;
    //      return true;
    //    }
    //    else if (binder.Name == "Avg") {
    //      result = (Func<object>) CalcAvg;
    //      return true;
    //    }
    //  }
    //  return base.TryGetMember(binder, out result);
    //}













    public object Sum( ) {
      return IsNumeric ? CalcSum( ) : null;
    }

    public object Avg( ) {
      return IsNumeric ? CalcAvg( ) : null;
    }

  }
}